{% extends "dashboard_app/_content.html" %}
{% load markup %}
{% load i18n %}


{% block title %}
{{ block.super }} | {% trans "Dashboard" %} | {% trans "API Help (deprecated)" %}
{% endblock %}


{% block extrahead %}
{{ block.super }}
<script type="text/javascript" src="{{ STATIC_URL }}lava-server/js/jquery-1.5.1.min.js"></script>
<script type="text/javascript" src="{{ STATIC_URL }}lava-server/js/jquery-ui-1.8.12.custom.min.js"></script>
<link rel="stylesheet" type="text/css" href="{{ STATIC_URL }}lava-server/css/Aristo/jquery-ui-1.8.7.custom.css"/>
{% endblock %}


{% block breadcrumbs %}
<li><a href="{% url dashboard_app.views.index %}">{% trans "Dashboard" %}</a></li>
<li><a href="{% url dashboard_app.views.dashboard_xml_rpc_handler %}">{% trans "API Help (deprecated)" %}</a></li>
{% endblock %}


{% block content %}
<h2>Depracation notice</h2>
<p>Dashboard has a separate, deprecated, XML-RPC handler. You should refrain
from using it. Instead please see the <a href="{% url lava.api_help %}"
  >LAVA Server API Help</a> which provides API services to all services</p>
{% endblock %}
