{% extends "layouts/content_with_sidebar.html" %}


{% block extrahead %}
{{ block.super }}
{% include "dashboard_app/_extrahead.html" %}
{% endblock %}


{% block title %}
{{ block.super }}{% include "dashboard_app/_title.html" %}
{% endblock %}


{% block breadcrumbs %}
{% include "dashboard_app/_breadcrumbs.html" %}
{% endblock %}


{% block navigation %}
{% include "dashboard_app/_extension_navigation.html" %}
{% endblock %}
